#
# Copyright (c) 2013
# Nathan Currier
#
# Use, modification, and distribution are all subject to the
# Boost Software License, Version 1.0. (See the accompanying
# file LICENSE.md or at http://rideliner.tk/LICENSE.html).
#
# <description>
#

class PathSearch
protected
  @defaultExts = [ ]
  @paths = [ ]
private
  def searchAllPaths filename
    base = File.basename filename
    paths.each { |dir|
      if File.exist?(File.join dir, base)
        return File.join dir, base
      end
    }

    nil
  end

  def searchWithExts filename
    defaultExts.each { |ext|
      file = "#{filename}.#{ext}"
      found = searchAllPaths file
      if found != nil
        return found
      end
    }

    nil
  end
public
  def search filename
    if File.exist? filename
      file = filename
    end

    if file == nil
      file = searchAllPaths File.basename(filename)
    end

    if file == nil
      file = searchWithExts File.basename(filename, File.extname(filename))
    end

    if block_given?
      if file != nil
        yield file
      else
        Bakery::Log.warning "no results found in search: #{filename}"
        Bakery::Log.debug "search path: #{paths}"
      end
    end

    file
  end

  def initialize paths = [ ]
    @paths = paths
  end

  def addSearchDirectory dir
    if File.exist?(dir) && File.directory?(dir) && !@paths.include?(dir)
      Bakery::Log.debug "directory added to search path: #{dir}"

      @paths << dir
    else
      Bakery::Log.warning "directory not added to search path: #{dir}"
    end
  end

  attr_accessor :paths, :defaultExts
end

